#############################################################
#> BEGIN DESCRIPTION
#> Package: grin
#> Type: Package
#> Title: Genomic Random Interval Modeling and Analysis
#> Version: 1.4
#> Author: Stan Pounds <stanley.pounds@stjude.org> Zhifa Liu <zhifa.liu@stjude.org>
#> Maintainer: Zhifa Liu <zhifa.liu@stjude.org>
#> Depends: R (>= 2.15.0), tools, MASS
#> Date: 2013-06-10
#> Description: Uses Genomic Random Intervals to Perform Functional Analysis of Genomic Lesion Data.
#> License: GPL-2
#> LazyData: yes
#> LazyLoad: yes  
#> URL: http://www.stjude.org/pounds
#> END DESCRIPTION

#############################################################
#> BEGIN NAMESPACE
#> exportPattern("grin.genes|grin.genome|grin.gsets|add.gene.anns|chrom.as.number|seg.heatmap|seg.man.plot|grin.analysis|hg19.gene.annotation.data|hg19.chrom.sizes.data|KEGG.gset.data|hg18.gene.annotation.data|hg18.chrom.sizes.data")
#> END NAMESPACE

#############################################################
#> BEGIN grin-package
#> \alias{grin-package}
#> \alias{grin}
#> \docType{package}
#> \title{Use genomic random interval (GRIN) modeling to identify particular loci, genes, or gene-sets with a significance abundance of genomic lesions.}
#> \description{Genomic random interval (GRIN) analysis attempts to infer the biological significance of a genomic lesion data set by statistically modeling the genomic 
#> lesions as a set of genomic random intervals.  Briefly, the statistical 
#> model represents lesions as genomic random intervals of fixed length that may occur at any position along the same
#> chromosome with uniform probability.  The frequencies at which lesions overlap with specific genomic loci, 
#> specific genes, or gene sets are statistics that measure the extent to which lesions `target' those loci, genes,
#> or gene sets.  The observed frequencies are compared to the probability distribution derived by modeling the lesions as genomic random intervals. 
#> A genomic random interval (GRIN) model is used as a null model to evaluate the statistical significance of the pattern of overlap of genomic lesions 
#> with the loci of individual genes, predefined sets of genes, and each base-pair locus in the genome.  The null model represents each lesion as an interval 
#> of fixed length and random location that may occur at any location along the chromosome with equal probability.  The null model defines null distributions 
#> for the total number of overlapping gene-lesion pairs, the number of lesions that overlap any locus in a gene set, and the number of tumors with a lesion 
#> that overlaps any locus in a gene set.  The significance of each overlap statistic may be determined by comparing its observed value to its null distribution 
#> defined by the GRIN model.}
#> \details{
#> \tabular{ll}{
#> Package: \tab grin\cr
#> Type: \tab Package\cr
#> Version: \tab 1.1\cr
#> Date: \tab 2013-07-18\cr
#> License: \tab GPL(>=2) \cr }
#> }
#> \author{Stan Pounds <stanley.pounds@stjude.org> Zhifa Liu <zhifa.liu@stjude.org>
#>
#> Maintainer: Zhifa Liu <zhifa.liu@stjude.org>
#> }
#> \references{Pounds S, Cheng C, Li, S, Liu Z, Zhang J, Mullighan C(2013) Genomic Random Interval Models for Statistical Analysis of Genomic Lesion Data. Bioinformatics.
#>
#>
#>             Pounds and Cheng (2006) Robust estimation of the false discovery rate. Bioinformatics 22: 1979-87.}
#> END grin-package


#############################################################
#> BEGIN hg18.chrom.sizes.data
#> \alias{hg18.chrom.sizes.data}
#> \docType{data}
#> \title{hg18 chromosome size data}
#> \description{Gives the size of each chromosome under the hg18 version of the human genome.}
#> \format{ A data frame with 25 observations for the following 2 variables.
#> \describe{
#>    \item{\code{chrom}}{a numeric vector with the chromosome.  The convention X=23, Y=24, MT=25 is used.}
#>    \item{\code{size}}{a numeric vector with the size of the chromosome in base-pairs.}
#>  } }
#> \examples{data(hg18.chrom.sizes.data)}
#> \keyword{datasets}
#> END hg18.chrom.sizes.data

#############################################################
#> BEGIN hg19.chrom.sizes.data
#> \alias{hg19.chrom.sizes.data}
#> \docType{data}
#> \title{hg19 chromosome size data}
#> \description{Gives the size of each chromosome under the hg19 version of the human genome.}
#> \format{ A data frame with 25 observations on the following 2 variables.
#> \describe{
#>    \item{\code{chrom}}{a numeric vector with the chromosome.  The convention X=23, Y=24, MT=25 is used.}
#>    \item{\code{size}}{a numeric vector with the size of the chromosome in base-pairs.}
#>  } }
#> \examples{data(hg19.chrom.sizes.data)}
#> \keyword{datasets}
#> END hg19.chrom.sizes.data

#############################################################
#> BEGIN hg18.gene.annotation.data
#> \alias{hg18.gene.annotation.data}
#> \docType{data}
#> \title{hg18.gene.annotation.data}
#> \description{
#> The gene annotation data set for hg 18
#> }
#> \format{ A data frame with 63 observations on the following 5 variables.
#> \describe{
#>     \item{\code{gene.id}}{a numeric vector with the NCBI gene identifier}
#>     \item{\code{gene.label}}{a character vector with the gene symbol}
#>     \item{\code{chrom}}{a numeric vector with the chromosome.  Uses the convention X=23, Y=24, MT=25.}
#>     \item{\code{loc.start}}{a numeric vector with the start locus of the gene.}
#>     \item{\code{loc.end}}{a numeric vector with the end locus of the gene.}
#>   }
#> }
#> \examples{
#> data(hg18.gene.annotation.data)
#> head(hg18.gene.annotation.data)
#> }
#> \keyword{datasets}
#> END hg18.gene.annotation.data


#############################################################
#> BEGIN hg19.gene.annotation.data
#> \alias{hg19.gene.annotation.data}
#> \docType{data}
#> \title{hg19.gene.annotation.data  }
#> \description{
#> The gene annotation data set for hg 19
#> }
#> \format{ A data frame with 63 observations on the following 5 variables.
#> \describe{
#>     \item{\code{gene.id}}{a numeric vector with the NCBI gene identifier}
#>     \item{\code{gene.label}}{a character vector with the gene symbol}
#>     \item{\code{chrom}}{a numeric vector with the chromosome.  Uses the convention X=23, Y=24, MT=25.}
#>     \item{\code{loc.start}}{a numeric vector with the start locus of the gene.}
#>     \item{\code{loc.end}}{a numeric vector with the end locus of the gene.}
#>   }
#> }
#> \examples{
#> data(hg19.gene.annotation.data)
#> head(hg19.gene.annotation.data)
#> }
#> \keyword{datasets}
#> END hg19.gene.annotation.data

#############################################################
#> BEGIN KEGG.gset.data
#> \alias{gset.data}
#> \docType{data}
#> \title{A example of a gene-set data set.}
#> \description{
#> This data set includes the locations of a few selected genes for example.  The selected genes are members of the 
#> KEGG acute myeloid leukemia pathway and a list of genes known to be important for T-cell development and GO.biological processes.
#> }
#> \format{A data frame with 1140 observations on the following 4 variables.
#>   \describe{
#>     \item{\code{gset.source}}{a character vector that provides the source of the gene-set definition (e.g., KEGG)}
#>     \item{\code{gset.id}}{a character vector that gives the identifier for the gene-set}
#>     \item{\code{gset.name}}{a character vector that gives the label (meaningful name) of the gene-set}
#>     \item{\code{gene.id}}{a character vector that gives the identifier of gene assigned to the gene-set.}
#>   }
#> }
#> \examples{
#> data(KEGG.gset.data)
#> head(KEGG.gset.data)
#> table(KEGG.gset.data$gset.id)
#> }
#> \keyword{datasets}
#> END KEGG.gset.data


#############################################################
#> BEGIN ETP.lesion.data
#> \alias{ETP.lesion.data}
#> \docType{data}
#> \title{lesion data observed in early T-cell precursor acute lymphoblastic leukemia.}
#> \description{Provides the locations of lesions observed in 12 early T-cell precursor leukemias.}
#> \format{
#>   A data frame with 402 observations on the following 5 variables.
#>   \describe{
#>     \item{\code{subj.ID}}{a character vector with the subject identifier}
#>     \item{\code{chrom}}{a numeric vector with the chromosome number, uses the convention X=23, Y=24, MT=25.}
#>     \item{\code{loc.start}}{a numeric vector with the start location of the lesion}
#>     \item{\code{loc.end}}{a numeric vector with the end location of the lesion}
#>     \item{\code{type}}{a vector indicating the lesion type such as gain, loss, or indel...}
#>   }
#> }
#> \references{
#> Zhang et al. The genetic basis of early T-cell precursor acute lymphoblastic leukaemia (2012). Nature, 481:157-63.
#> }
#> \examples{
#> data(ETP.lesion.data)
#> head(ETP.lesion.data)
#> }
#> \keyword{datasets}
#> END ETP.lesion.data

#############################################################
#> BEGIN HYPO.lesion.data
#> \alias{HYPO.lesion.data}
#> \docType{data}
#> \title{lesion data observed in hypodiploid acute lymphoblastic leukemia}
#> \description{Provides the locations of lesions observed in hypodiploid acute lymphoblastic leukemia.}
#> \format{
#>   A data frame with 940 observations on the following 5 variables.
#>   \describe{
#>     \item{\code{subj.ID}}{a character vector with the subject identifier}
#>     \item{\code{chrom}}{a numeric vector with the chromosome number, uses the convention X=23, Y=24, MT=25.}
#>     \item{\code{loc.start}}{a numeric vector with the start location of the lesion}
#>     \item{\code{loc.end}}{a numeric vector with the end location of the lesion}
#>     \item{\code{type}}{a vector indicating the lesion type such as gain, loss, or indel...}
#>   }
#> }
#> \references{
#> Holmfeldt et al. The genomic landscape of hypodiploid acute lymphoblastic leukemia (2013). Nature Genetics,45:242-252.
#> }
#> \examples{
#> data(HYPO.lesion.data)
#> head(HYPO.lesion.data)
#> }
#> \keyword{datasets}
#> END HYPO.lesion.data


#############################################################
#> BEGIN grin.gsets
#> \title{gene set analysis}
#> \description{Perform GRIN analysis for overlap of lesions with pre-defined gene-sets}
                                                          #> \arguments{
grin.gsets<-function(lesion.locs,                         #> \item{lesion.locs}{data.frame with one row per lesion, columns for subj.ID, chrom, loc.start, loc.end}
                     gene.locs=hg19.gene.annotation.data, #> \item{gene.locs}{data.frame with one row per gene, columns for gene.id, gene.label, chrom, loc.start, loc.end. The default is hg19.gene.annotation.data}
                     gset.data=KEGG.gset.data,            #> \item{gset.data}{data.frame with one row per assignment of gene to gene-set, columns for gset.id, gset.label, and gene.id.The default is KEGG.gset.data}
                     chrom.sizes=hg19.chrom.sizes.data,   #> \item{chrom.sizes}{data.frame with one row per chromosome, columns for chrom and size. The default is hg19.chrom.sizes.data}
                     edge.adjustment=F,                   #> \item{edge.adjustment}{indicates whether to circularize chromosomes to eliminate edge effects}
                     remove.dups=T,                       #> \item{remove.dups}{indicates whether to remove duplicate entries from gene.locs and gset.data before analysis}
                     min.nhits.report=1,                  #> \item{min.nhits.report}{minimum number of hits to include in the report}
                     order.results=T,                     #> \item{order.results}{indicates whether to order results by cohort.max.pvalue in ascending order}
                     quiet=F,                             #> \item{quiet}{indicates whether to be quiet while running, if FALSE it will output progress messages}
                     compute.q=T)                         #> \item{compute.q}{used to compute the q value:the smoothed local False Discovery Rate estimates}
                                                          #> }
{
  if (!all(is.element(c("subj.ID","chrom","loc.start","loc.end"),colnames(lesion.locs)))) stop("lesion.locs must have the subj.ID, chrom, loc.start, and loc.end columns.") 
  if (!all(is.element(c("gene.id","gene.label","chrom","loc.start","loc.end"),colnames(gene.locs)))) stop("gene.locs must have the gene.id, gene.label, chrom, loc.start, and loc.end columns.")
  if (!all(is.element(c("chrom","size"),colnames(chrom.sizes)))) stop("chrom.sizes must have the chrom and size columns.")
  if (!all(is.element(c("gset.id","gene.id"),colnames(gset.data)))) stop("gset.data must have the gset.id and gene.id columns.")
  if(remove.dups)
   {
      dup.gloc<-duplicated(gene.locs$gene.id)
      gene.locs<-gene.locs[!dup.gloc,]

      dup.gset<-duplicated(paste(gset.data$gset.id,gset.data$gene.id,sep="_"))
      gset.data<-gset.data[!dup.gset,]
   }

   if(is.null(gene.locs$gene.label)) gene.locs$gene.label<-as.character(gene.locs$gene.id)
   #rownames(gene.locs)<-gene.locs$gene.label

   gset.anns<-gset.annotations(gset.data,gene.locs)
   n.gset<-dim(gset.anns)[1]

   uniq.subjID<-sort(unique(lesion.locs$subj.ID))
   n.subj<-length(uniq.subjID)


   cohort.res<-NULL
   subj.res<-NULL
   compute.time<-matrix(NA,n.gset,3)
   options(stringsAsFactors=FALSE)
   for (i in 1:n.gset)  # Loop over gene-sets
   {
       if(!quiet) message(paste("Working on gene set",gset.anns$gset.id[i],"(",i,"of",n.gset,"):",date()))
       t1<-proc.time()
       # subset on gene locations for this gene-set
       this.gset<-is.element(gset.data$gset.id,gset.anns$gset.id[i])
       gset.locs<-is.element(gene.locs$gene.id,gset.data$gene.id[this.gset])
       ##print(table(gset.locs))

       # perform the analysis
       gset.res<-compute.hit.pvals.one.gset(lesion.locs,gene.locs[gset.locs,],chrom.sizes,
                                            edge.adjustment=edge.adjustment)
       #browser()
       # populate cohort-level results into the result data.frame
       cohort.nhits.subj<-subj.nhits.string(gset.res$subj.res,min.nhits.report)
       cohort.nhits.gene<-gene.nhits.string(gset.res$subj.res,min.nhits.report)
       cohort.res.vec<-c(cohort.nhits.subj=cohort.nhits.subj,
                         cohort.nhits.gene=cohort.nhits.gene,
                         gset.res$cohort.res)

       cohort.res<-rbind.data.frame(cohort.res,cohort.res.vec,stringsAsFactors=FALSE)

       # populate subject-level results into the result data.frame
       subj.res.vec<-subj.results.vector(gset.res$subj.res)
       subj.res<-rbind.data.frame(subj.res,subj.res.vec,stringsAsFactors=FALSE)
       t2<-proc.time()
       compute.time[i,]<-(t2-t1)[1:3]
    
   }
   colnames(subj.res)<-names(subj.res.vec)
   colnames(cohort.res)<-names(cohort.res.vec)
   colnames(compute.time)<-paste("compute.time",names(t2)[1:3],sep=".")
   subj.res.stat.cols<-c(grep("nlesions",colnames(subj.res)),grep("noverlap",colnames(subj.res)))
   subj.res[,subj.res.stat.cols]<-as.numeric(unlist(subj.res[,subj.res.stat.cols]))
   cohort.res = cohort.res [seq(from=1,to=dim(cohort.res)[1],by=2),] # the even rows should be removed
   subj.res = subj.res [seq(from=1,to=dim(subj.res)[1],by=2),]
   comb.res<-cbind.data.frame(gset.anns,cohort.res,subj.res,compute.time)
  
   if (order.results)
   {
     min.pval=apply(comb.res[,c("p.overlaps","p.lesions","p.subjects")],1,min)
     ord<-order(min.pval)
     comb.res<-comb.res[ord,]
   }
   if (compute.q==T & n.gset>5)
   {
   comb.res$q.subjects <- robust.fdr(as.numeric (comb.res$p.subjects),sides=2,discrete=F)$q
   comb.res$q.overlaps <- robust.fdr(as.numeric(comb.res$p.overlaps),sides=2,discrete=F)$q 
   comb.res$q.lesions <- robust.fdr(as.numeric(comb.res$p.lesions),sides=2,discrete=F)$q
   }
   return(comb.res)
}

#> \examples{
#> data(ETP.lesion.data)
#> data(hg18.gene.annotation.data)
#> data(hg18.chrom.sizes.data)
#> data(KEGG.gset.data)
#> #gset.res<-grin.gsets(ETP.lesion.data, hg18.gene.annotation.data, KEGG.gset.data, hg18.chrom.sizes.data) #commented out example
#> }
#> \references{Pounds S, Cheng C, Li, S, Liu Z, Zhang J, Mullighan C(2013) Genomic Random Interval Models for Statistical Analysis of Genomic Lesion Data. Bioinformatics.
#>
#>
#>             Pounds and Cheng (2006) Robust estimation of the false discovery rate. Bioinformatics 22: 1979-87.}

#> \value{a list with the following main components:
#>            \item{gset.source}{lists the source of the gene set definition}
#>            \item{gset.id}{lists the identifier of the gene set}
#>            \item{gset.label}{lists the label of the gene set}
#>            \item{ngenes.gset}{gives the number of genes assigned to the gene set}
#>            \item{gset.gene.ids}{lists the genes Entry ID(NCBI) assigned to the gene set}
#>            \item{ngenes.with.locus.data}{gives the number of genes in the set that were successfully matched to the gene location data}
#>            \item{all.genes.matched}{indicates (TRUE/FALSE) whether all genes in the set were successfully matched to the gene location data}
#>            \item{n.overlaps}{total number of lesion loci that overlap the locus of the gene set}
#>            \item{p.overlaps}{ p-value for n.overlaps from the convolution model}
#>            \item{q.overlaps}{false discovery rate for p.overlaps}
#>            \item{n.subjects}{lists the number of subjects with a lesion overlapping the interval.}
#>            \item{p.subjects}{p-value for n.subjects from the convolution model}
#>            \item{q.subjects}{false discovery rate for p.subjects} 
#>            \item{n.lesions}{the total number of lesions that overlap at least one gene in the gene set}
#>            \item{p.lesions}{p-value for n.lesion from the convolution model}
#>            \item{q.lesions}{false discovery rate for p.lesions}  
#>           }

#> END grin.gsets                 


#############################################################
#> BEGIN grin.genes
#> \title{gene analysis}
#> \description{perform GRIN analysis for individual genes}
                                                           #> \arguments{
grin.genes<-function(lesion.locs,                          #> \item{lesion.locs}{data.frame with one row per lesion, required columns are subj.ID, chrom, loc.start, and loc.end}
                     gene.locs=hg19.gene.annotation.data,  #> \item{gene.locs}{data.frame with one row per gene, columns for gene.id, gene.label, chrom, loc.start, loc.end. The default is hg19.gene.annotation.data}
                     chrom.sizes=hg19.chrom.sizes.data,    #> \item{chrom.sizes}{data.frame with one row per chromosome, required columns are chrom & size. The default is hg19.chrom.sizes.data}
                     compute.q=T,                          #> \item{compute.q}{used to compute the q value:the smoothed local False Discovery Rate estimates}
                     edge.adjustment=T)                    #> \item{edge.adjustment}{indicates whether to circularize chromosomes to eliminate edge effects} 
                                                           #> }
{
   lesion.locs$subj.ID<-as.character(lesion.locs$subj.ID)          # convert subject ID to a character string
   usubj<-sort(unlist(unique(lesion.locs$subj.ID)))                # Extract and Order subject identifiers
   nsubjects<-length(usubj)                                        # determine the number of subjects
   ngenes<-dim(gene.locs)[1]                                       # determine number of genes
   nlesions<-dim(lesion.locs)[1]                                   # total number of observed lesions
   noverlap<-matrix(NA,ngenes,nsubjects)                           # Initialize subject specific noverlap matrix
   P.nsubj<-P.nlesion<-matrix(1,ngenes,1)                          # Initialize matrices to have probability 1 of 0 overlaps
   lesion.locs$size<-lesion.locs$loc.end-lesion.locs$loc.start+1   # compute lesion size
   nsubj<-nlesion<-rep(0,ngenes)                                   # Initialize observed value
   
   colnames(noverlap)<-usubj                                       # Columns of noverlap matrix correspond to subjects
   
   

   for (i in 1:nsubjects)   # Loop over subjects
   {
      #message(paste("Computing GRIN model probabilities subject",i,"of",nsubjects,":",date()))
      subj.mtch<-is.element(lesion.locs$subj.ID,usubj[i]) # identify lesions for subject i
      subj.data<-lesion.locs[subj.mtch,]                  # subset lesion data on subject i
      nlesion.subject<-sum(subj.mtch)                     # number of lesions for subject i
      noverlap.subject<-rep(0,ngenes)                     # initialize vector of number of lesions overlapping each gene for this subject
      P.subj<-matrix(1,ngenes,1)                          # initialize GRIN probability to probability of no overlaps is 1
    
      
      for (j in 1:nlesion.subject)                        # Loop over lesions for this subject
      {
         message(paste(" Working on lesion",j,"of",nlesion.subject,"for subject",usubj[i], " ", i,":",date()))
         chr.mtch<-is.element(chrom.sizes$chrom,subj.data$chrom[j])                 # identify the chromosome of this lesion
         gene.on.chrom<-is.element(gene.locs$chrom,subj.data$chrom[j])              # indicates which genes are on the same chromosome as this lesion    
         if (edge.adjustment)
         {
          p=(gene.locs$loc.end-gene.locs$loc.start+1+subj.data$size[j])*gene.on.chrom/chrom.sizes$size[chr.mtch]
          p[p>1]=1
         }
         else
         {
          a<-pmax(1,gene.locs$loc.start-subj.data$size[j]+1)                         # smallest start location for this lesion to overlap each gene
          b<-pmin(chrom.sizes$size[chr.mtch]-subj.data$size[j],gene.locs$loc.end)    # largest start location for this lesion to overlap each gene

         
          p<-(b-a+1)*gene.on.chrom/(chrom.sizes$size[chr.mtch]-subj.data$size[j]+1)  # GRIN model probability this lesion overlaps each gene
         }

 

         P.temp<-cbind(1-p,p)                                                       # create GRIN probability matrix for row.convolution
         P.subj<-row.convolution(P.subj,P.temp)                                     # update convolution for this subject
        
        
         # determine whether each gene overlaps the lesion
         on.chrom<-is.element(gene.locs$chrom,subj.data$chrom[j])                   # determine which genes are on the same chromosome as this lesion
         overlap<-(subj.data$loc.start[j]<=gene.locs$loc.end[on.chrom])*            # determine which genes overlap with this lesion
                  (subj.data$loc.end[j]>=gene.locs$loc.start[on.chrom])

         # update the number of overlaps per gene for this subject
         noverlap.subject[on.chrom]<-noverlap.subject[on.chrom]+overlap
      }
      message(paste(" Updating overlap statistics:",date()))
      nsubj<-nsubj+(noverlap.subject>0)
      nlesion<-nlesion+noverlap.subject
      noverlap[,i]<-noverlap.subject

      message(paste(" Updating convolutions for subject",i,"of",nsubjects,":",date()))
      P.nlesion<-row.convolution(P.subj,P.nlesion)  # update convolution for total number of lesions overlapping the gene


      p.subj<-1-P.subj[,1]                           # probability that at least one lesion overlaps each gene for this subject
      p.subj<-cbind(1-p.subj,p.subj)                 # create matrix for row.convolution
      P.nsubj<-row.convolution(p.subj,P.nsubj)       # update convolution for total number of subjects with a lesion overlapping each gene
      


   }
  
   message(paste("Comparing overlap stats to convolution distributions:",date()))
   p.nsubj<-p.nlesion<-rep(NA,ngenes)

   ncol.Pnsubj<-dim(P.nsubj)[2]
   ncol.Pnles<-dim(P.nlesion)[2]


   for (i in 1:ngenes) # Loop over genes to compute p-value for each gene
   {
     p.nsubj[i]<-sum(P.nsubj[i,(nsubj[i]+1):ncol.Pnsubj])
     p.nlesion[i]<-sum(P.nlesion[i,(nlesion[i]+1):ncol.Pnles])
   }
   
   
   pval.mtx<-cbind(p.nsubj,p.nlesion)
   min.pval<-apply(pval.mtx,1,min)
   options(stringsAsFactors=F)
   result<-cbind.data.frame(gene.locs,
                            n.overlaps=nlesion,
                            p.overlaps=p.nlesion,
                            n.subjects=nsubj,
                            p.subjects=p.nsubj,
                            min.p=min.pval,
                            noverlap)
                            
   ord<-order(result$min.p)
   result<-result[ord,]
   
   if (compute.q==T)
   {
   result$q.subjects <- robust.fdr(result$p.subjects,sides=2,discrete=F)$q
   result$q.overlaps <- robust.fdr(result$p.overlaps,sides=2,discrete=F)$q
   result.genome.col=c(colnames(result)[1:5],"n.overlaps","p.overlaps","q.overlaps","n.subjects","p.subjects","q.subjects","min.p",unique(lesion.locs$subj.ID))
   result.genome.ord=order(result$p.subjects) 
   result <- result[result.genome.ord,result.genome.col]
   }
   else
   {
   result.genome.col=c(colnames(result)[1:5],"n.overlaps","p.overlaps","n.subjects","p.subjects","min.p",unique(lesion.locs$subj.ID))
   result.genome.ord=order(result$p.subjects) 
   result <- result[result.genome.ord,result.genome.col]
   }
   return(result)
}
#> \examples{
#> data(ETP.lesion.data)
#> data(hg18.gene.annotation.data)
#> data(hg18.chrom.sizes.data)
#> #gene.res<-grin.genes(ETP.lesion.data, hg18.gene.annotation.data, hg18.chrom.sizes.data)
#> }
#> \references{Pounds S, Cheng C, Li, S, Liu Z, Zhang J, Mullighan C(2013) Genomic Random Interval Models for Statistical Analysis of Genomic Lesion Data. Bioinformatics.
#>
#>
#>             Pounds and Cheng (2006) Robust estimation of the false discovery rate. Bioinformatics 22: 1979-87.}

#> \value{a list with the following components:
#>            \item{chrom}{chrom number}
#>            \item{gene.id}{NCBI Entry gene ID}
#>            \item{gene.label}{gene name}
#>            \item{loc.start}{location of start of each gene }
#>            \item{loc.end}{location of end of each gene}
#>            \item{n.overlaps}{total number of lesion loci that overlap the locus of the gene}
#>            \item{p.overlaps}{ p-value for n.overlaps from the convolution model}
#>            \item{q.overlaps}{false discovery rate for p.overlaps}
#>            \item{n.subjects}{lists the number of subjects with a lesion overlapping the interval.}
#>            \item{p.subjects}{p-value for n.subjects from the convolution model}
#>            \item{q.subjects}{false discovery rate for p.subjects} 
#>           }
#> END grin.genes


#############################################################
#> BEGIN grin.genome
#> \title{grin.gemome}
#> \description{Perform GRIN analysis for genomic interval loci defined by lesion and chromosome endpoints}
                                                             #> \arguments{
grin.genome <- function(lesion.locs,                         #> \item{lesion.locs}{data.frame with one row per lesion, required columns are subj.ID, chrom, loc.start, loc.end}
                        gene.locs=hg19.gene.annotation.data, #> \item{gene.locs}{data.frame with one row per gene, columns for gene.id, gene.label, chrom, loc.start, loc.end. The default is hg19.gene.annotation.data }
                        chrom.sizes=hg19.chrom.sizes.data,   #> \item{chrom.sizes}{data.frame with one row per chromosome, required columns are chrom & size. The default is hg19.chrom.sizes.data }
                        edge.adjustment=T,                   #> \item{edge.adjustment}{indicates whether to circularize chromosomes to eliminate edge effects}
                        compute.q=T)                         #> \item{compute.q}{used to compute the q value:the smoothed local False Discovery Rate estimates}
                                                             #> }
{
   # determine data-defined interval loci
   uniq.ints<-lesion.matrix(lesion.locs,chrom.sizes)      
   # prepare data.frame of "gene.locs" for grin.genes
   int.locs<-cbind.data.frame(interval.id=1:(dim(uniq.ints)[1]),
                              chrom=uniq.ints$chrom,
                              loc.start=uniq.ints$loc.start,
                              loc.end=uniq.ints$loc.end,
                              size=uniq.ints$loc.end-uniq.ints$loc.start+1)
   # use grin.genes for analysis
   res<-grin.genes(lesion.locs,int.locs,chrom.sizes,edge.adjustment=edge.adjustment,compute.q=F)
   res<- add.gene.anns(res,2,3,4,gene.locs,"gene.label","chrom","loc.start","loc.end")
   if (compute.q==T)
   {
   res$q.subjects <- robust.fdr(res$p.subjects,sides=2,discrete=F)$q
   res$q.overlaps <- robust.fdr(res$p.overlaps,sides=2,discrete=F)$q
   res.genome.col=c(colnames(res)[1:5],"n.gene","gene.list","n.overlaps","p.overlaps","q.overlaps","n.subjects","p.subjects","q.subjects",unique(lesion.locs$subj.ID))
   res.genome.ord=order(res$p.subjects) 
   res <- res[res.genome.ord,res.genome.col]
   }
   else
   {
   res.genome.col=c(colnames(res)[1:5],"n.gene","gene.list","n.overlaps","p.overlaps","n.subjects","p.subjects",unique(lesion.locs$subj.ID))
   res.genome.ord=order(res$p.subjects) 
   res <- res[res.genome.ord,res.genome.col]
   }
   return(res)
}
#> \examples{
#> data(ETP.lesion.data)
#> data(hg18.gene.annotation.data)
#> data(hg18.chrom.sizes.data)
#> #genome.res<-grin.genome(ETP.lesion.data, hg18.gene.annotation.data,hg18.chrom.sizes.data)
#> }
#> \references{Pounds S, Cheng C, Li, S, Liu Z, Zhang J, Mullighan C(2013) Genomic Random Interval Models for Statistical Analysis of Genomic Lesion Data. Bioinformatics.
#>
#>
#>             Pounds and Cheng (2006) Robust estimation of the false discovery rate. Bioinformatics 22: 1979-87.}
#> \value{a list with the following components:
#>            \item{interval.id}{gives a numeric identifier to the interval}
#>            \item{chrom}{chrom number}
#>            \item{loc.start}{location of start of each gene}
#>            \item{loc.end}{location of end of each gene}
#>            \item{size}{gives the size of the interval in base pairs}
#>            \item{n.gene}{gives the number of genes in the segment}
#>            \item{gene.list}{lists the gene names in the segment}
#>            \item{n.overlaps}{total number of lesion loci that overlap the locus of the gene}
#>            \item{p.overlaps}{ p-value for n.overlaps from the convolution model}
#>            \item{q.overlaps}{false discovery rate for p.overlaps}
#>            \item{n.subjects}{lists the number of subjects with a lesion overlapping the interval}
#>            \item{p.subjects}{p-value for n.subjects from the convolution model}
#>            \item{q.subjects}{false discovery rate for p.subjects} 
#>           }
#> END grin.genome


############################################################
# Function: row.convolution
# Purpose: compute convolution for number of hits for each gene
# Arguments: P1 - a matrix with one row per gene, columns give probability for number of overlapping lesions
#            P2 - a matrix with one row per gene, columns give probability for number of overlapping lesions
# Returns: a matrix with one row per gene and columns have convolution probability for number of overlapping lesions
############################################################

                              
row.convolution<-function(P1,P2)            

{
   g1<-dim(P1)[1]
   g2<-dim(P2)[1]

   if (g1!=g2) stop("P1 and P2 must have the same number of rows.")

   n1<-dim(P1)[2]-1  # maximum number of overlaps for distribution 1
   n2<-dim(P2)[2]-1  # maximum number of overlaps for distribution 2

   n3<-n1+n2  # maximum number of overlaps for convolution of distributions

   P3<-matrix(NA,g1,n3+1)  # initialize convolution result

   for (i in 0:n3) # Loop over possible columns for P3
   {
      # Get indices of columns of P1 and P2 that contribute to column i of the convolution
      num1<-(0:n3)  #
      num2<-i-num1
      ok.nums<-(num1<=n1)&(num2>=0)&(num2<=n2)
      num1<-num1[ok.nums]
      num2<-num2[ok.nums]
      # Now compute the probabilities for this column of P3
      if (length(num1)==1) P3[,i+1]<-exp(log(P1[,num1+1])+log(P2[,num2+1]))  # only one column from P1 & one column from P2 contribute
      if (length(num1)>1)
      {
         P1.star<-matrix(log(P1[,num1+1]),g1,length(num1))  # use exp(log(P)) for numerical stability,
         P2.star<-matrix(log(P2[,num2+1]),g2,length(num2))  # limit to columns of P1 & P2 that contribute to this column of P3
         P.prod<-exp(P1.star+P2.star)  # Get products
         sum.prod<-rowSums(P.prod)     # Get sums of products
         P3[,i+1]<-sum.prod            # Assign to this column of P3
         rm(P1.star,P2.star,P.prod,sum.prod)  # Clean up memory
         gc()
      }
   }

   # Eliminate unnecessary columns that have probability 0 for all genes
   cs<-colSums(P3)
   if (any(is.element(cs,0)))
   {
     k.max<-max((1:n3)[colSums(P3)>0])
     P3<-P3[,1:k.max]
   }

   # Return result
   return(P3)
}


#####################################
# Function: lesion.matrix
# Purpose: Represent lesion.locs in a matrix format
# Arguments: lesion.locs - data.frame with one row per lesion, required columns are subj.ID, chrom, loc.start, loc.end
#            chrom.sizes - data.frame with one row per chromosome, required columns are chrom and size
# Returns: a data.frame with one row per genomic region defined by endpoints of lesion.locs, columns for chrom, loc.start, loc.end, and individual result
###########################################


lesion.matrix<-function(lesion.locs,
                        chrom.sizes) 
{
   uniq.ints<-Unique.intervals(lesion.locs,chrom.sizes)     # determine unique intervals defined by lesion & chromosome endpoints
   uniq.ints$size<-uniq.ints$loc.end-uniq.ints$loc.start+1  # compute size of the unique intervals

   usubj<-sort(unlist(unique(as.character(lesion.locs$subj.ID))))  # get list of unique subject identifiers in lesion.locs
   nsubj<-length(usubj)                                            # number of subjects in lesion.locs
   n.ints<-dim(uniq.ints)[1]                                       # number of unique intervals
   lsn.mtx<-matrix(0,n.ints,nsubj)                                 # initialize lesion matrix with number of hits per interval for each subject

   n.lesion<-dim(lesion.locs)[1]                                   # number of lesions
   for (i in 1:n.lesion)  # loop over lesions
   {
      chr.mtch<-is.element(uniq.ints$chrom,lesion.locs$chrom[i])   # identify intervals located on the same chromosome as lesion i
      pos.mtch<-(uniq.ints$loc.end>=lesion.locs$loc.start[i])&     # identify intervals that overlap with lesion i
                (uniq.ints$loc.start<=lesion.locs$loc.end[i])
      subj.ind<-which(usubj==lesion.locs$subj.ID[i])               # get the column index for the subject with lesion i
      lsn.mtx[,subj.ind]<-lsn.mtx[,subj.ind]+(chr.mtch*pos.mtch)   # update the column for the subject with lesion i
   }
   colnames(lsn.mtx)<-usubj                                        # name the columns of lsn.mtx after the subjects
   res<-cbind.data.frame(uniq.ints,lsn.mtx)                        # combine the interval loci with the lesion overlap matrix
   res<-reduce.lesion.matrix(res)                                  # combine rows with identical results
   return(res)
}

####################################################
# Function: reduce.lesion.matrix
# Purpose: combine rows of lsn.mtx that have identical results
# Arguments: lsn.mtx, the result of lesion.matrix
# Result: a data.frame like lsn.mtx with the rows combined
#########################################

reduce.lesion.matrix<-function(lsn.mtx)  

{
   m<-dim(lsn.mtx)[1]        # Number of rows of lsn.mtx
   if (m==1) return(lsn.mtx) # If already just one row, there is nothing to do
   k<-dim(lsn.mtx)[2]        # Number of columns of lsn.mtx
   subj.ind<-(which(colnames(lsn.mtx)=="size")+1):k  # Subject-specific data begins on the column after size
   same.chr<-lsn.mtx$chrom[1:(m-1)]==lsn.mtx$chrom[2:m]  # Indicates whether two consecutive rows are on the same chromosome
   if(length(subj.ind)>1) same.patt<-rowSums(abs(lsn.mtx[1:(m-1),subj.ind]-lsn.mtx[2:m,subj.ind]))==0 # Indicates whether two consecutive rows have the same pattern of loss
   else same.patt<-diff(lsn.mtx[,subj.ind])==0  # For the special case of only one subject
   orig.index<-(1:m)  # numeric row indices of the input matrix
   interval.number<-cumsum(c(1,(!same.chr)|(!same.patt)))  # assign each row to a new interval that combines consecutive rows with the same pattern
   interval.start.index<-c(1,(2:m)[diff(interval.number)==1]) # start index is the first row with the interval number
   interval.end.index<-c(interval.start.index[2:length(interval.start.index)]-1,m) # end index is the last row with the interval number

   # create result data frame
   new.res<-cbind.data.frame(interval.number=1:length(interval.start.index),
                             chrom=lsn.mtx$chrom[interval.start.index],
                             loc.start=lsn.mtx$loc.start[interval.start.index],
                             loc.end=lsn.mtx$loc.end[interval.end.index],
                             size=lsn.mtx$loc.end[interval.end.index]-lsn.mtx$loc.start[interval.start.index]+1,
                             lsn.mtx[interval.start.index,subj.ind])
   return(new.res)
}


##########################################
# Function: Unique.intervals
# Purpose: Determine the genomic interval loci defined by lesion and chromosome endpoints
# Arguments: lesion.locs - data.frame with one row per lesion, required columns are subj.ID, chrom, loc.start, loc.end
#            chrom.sizes - data.frame with one row per chromosome, required columns are chrom and size
# Returns: a data.frame with one row per genomic interval locus with columns chrom, loc.start, and loc.end
##############################################

Unique.intervals<-function(lesion.locs,chrom.sizes)

{
  res<-NULL
  uchr<-unique(chrom.sizes$chrom)
  for (chr in uchr)
  {
     lesion.on.chr<-is.element(lesion.locs$chrom,chr)
     if (any(lesion.on.chr))
     {
        chr.lesions<-lesion.locs[lesion.on.chr,]
        brkpts<-c(1,unique(chr.lesions$loc.start),
                  unique(chr.lesions$loc.end),
                  chrom.sizes$size[chrom.sizes$chrom==chr])
        brkpts<-sort(unique(brkpts))
        nbrks<-length(brkpts)
        chr.ints<-rbind.data.frame(cbind.data.frame(chrom=chr,loc.start=brkpts,loc.end=brkpts), # represent boundaries as 1-base intervals
                                   cbind.data.frame(chrom=chr,loc.start=brkpts[1:(nbrks-1)]+1,loc.end=brkpts[2:nbrks]-1)) # get the intervals between boundaries
     }
     else chr.ints<-cbind.data.frame(chrom=chr,loc.start=1,loc.end=chrom.sizes$size[chrom.sizes$chrom==chr])
     res<-rbind.data.frame(res,chr.ints)
  }

  exc<-(res$loc.end<res$loc.start)
  res<-res[!exc,]
  ord<-order(res$chrom,res$loc.start,res$loc.end)
  res<-res[ord,]
  return(res)
}

######################################
# Function: gene.nhits.string
# Purpose: Generate string of the number of hits per subject for annotation of grin.gsets results
# Arguments: subj.res
#            min.nhits.report - minimum number of hits to report
# Returns: the character string with the number of hits per subject
###################################### 

gene.nhits.string<-function(subj.res,min.nhits.report)

{
   kcol<-dim(subj.res)[2]
   exc.col<-c(1:3,which(is.element(colnames(subj.res),c("p.overlaps","p.lesions"))))
   nsubj<-dim(subj.res)[1]
   ngene<-dim(subj.res)[2]-length(exc.col)
   subj.ghit.mtx<-subj.res[,-exc.col]
   subj.ghit.mtx<-matrix(as.numeric(unlist(subj.ghit.mtx)),nsubj,ngene)
   colnames(subj.ghit.mtx)<-colnames(subj.res)[-exc.col]
   nhits<-colSums(subj.ghit.mtx)
   gene.label<-colnames(subj.ghit.mtx)

   keep<-nhits>=min.nhits.report
   nhits<-nhits[keep]
   gene.label<-gene.label[keep]
   ord<-rev(order(nhits))
   nhits<-nhits[ord]
   gene.label<-gene.label[ord]
   gene.hit.str<-paste(paste(nhits,gene.label),collapse=", ")
   return(gene.hit.str)
}

#############################################
# Function: subj.nhits.string
# Purpose: create a string to annotate the results of grin.gsets with number of hits per subject
# Function: subj.res - a data.frame with columns "subj.ID" and "tot.overlaps" that gives the number of overlaps per subject
#           min.nhits.report - minimum number of overlaps to report
# Returns: a character string with the number of hits per subject in descending order
###################################################

subj.nhits.string<-function(subj.res,min.nhits.report)

{
       subj.nov.data<-subj.res[,c("subj.ID","n.overlaps")]  # Keep only the columns of interest
       subj.nov.keep<-subj.nov.data[,2]>=min.nhits.report     # Keep only the rows with at least min.nhits.report overlaps
       subj.nov.data<-subj.nov.data[subj.nov.keep,]             
       subj.nov.ord<-rev(order(subj.nov.data[,2]))            # Arrange in descending order of the number of overlaps
       subj.nhit.string<-paste(paste(subj.nov.data[subj.nov.ord,2],subj.nov.data[subj.nov.ord,1]),collapse=", ") # Generate the string
       return(subj.nhit.string)
}

###############################################
# Function: subj.results.vector
# Purpose:
# Arguments: subj.res - a data.frame with subject-specific results
# Returns:
###################################################

subj.results.vector<-function(subj.res)

{
    kcol<-dim(subj.res)[2]
    pval.cols<-grep("p",names(subj.res))
    exc.col<-c(1:3,pval.cols)

    nsubj<-dim(subj.res)[1]
    subj.ghit.string<-rep("0",nsubj)
    gene.labels<-unlist(colnames(subj.res)[-exc.col])
    for (i in (1:nsubj)[subj.res$n.overlaps>0])
    {
      nhit<-unlist(subj.res[i,-exc.col])
      names(nhit)<-gene.labels
      nhit<-nhit[nhit>0]
      ord<-rev(order(nhit))
      nhit<-nhit[ord]
      subj.ghit.string[i]<-paste(paste(nhit,names(nhit)),collapse=", ")
   }
   subj.res$gene.hits<-subj.ghit.string

   subj.result<-subj.res[,c(2:3,kcol+1,pval.cols)]

   colnames(subj.result)<-c("nlesions","noverlaps","gene.hits",colnames(subj.res)[pval.cols])

   res.vector<-as.vector(unlist(t(subj.result)))
   res.names<-rep("",length(res.vector))
   k.new<-dim(subj.result)[2]
   for (i in 1:nsubj) res.names[k.new*(i-1)+1:k.new]<-paste(subj.res$subj.ID[i],colnames(subj.result),sep=".")
   names(res.vector)<-res.names
   return(res.vector)
}

##########################################
# Function: gset.annotations
# Purpose: Annotate grin.gset results with number of genes, gene ids, etc.
# Arguments:  gset.data - data.frame defining gene-sets with one row per gene-to-gset map, requred columns are gset.id and gene.id
#             gene.locs - data.frame with one row per gene, required columns are gene.id, chrom, loc.start, and loc.end
# Returns: a data.frame with one row per gene.set, original columsn of gset.data and computed annotation columns
##################################################################

gset.annotations<-function(gset.data,gene.locs)

{
   if(is.null(gset.data$gset.label)) gset.data$gset.label<-paste("gene_set",gset.data$gset.id)
   if(is.null(gset.data$gset.source)) gset.data$gset.source<-"NA"
   gset.id.labels<-gset.data[!duplicated(gset.data$gset.id),]
   uniq.gset<-gset.id.labels$gset.id
   n.gset<-length(uniq.gset)

   ##print(gset.id.labels)
   gset.anns<-cbind.data.frame(gset.source=gset.id.labels$gset.source,
                               gset.id=gset.id.labels$gset.id,
                               gset.label=gset.id.labels$gset.label,
                               ngenes.gset=NA,
                               gset.gene.ids=NA,
                               ngenes.with.locus.data=NA,
                               genes.with.locus.data=NA,
                               all.genes.matched=F,
                               unmatched.genes="NA")

   for (i in 1:n.gset)
   {
      this.gset<-is.element(gset.data$gset.id,gset.anns$gset.id[i])
      gset.anns$ngenes.gset[i]<-length(unique(gset.data$gene.id[this.gset]))
      gset.anns$gset.gene.ids[i]<-paste(gset.data$gene.id[this.gset],collapse=" // ")

      locs.in.gset<-is.element(gene.locs$gene.id,gset.data$gene.id[this.gset])
      gset.anns$ngenes.with.locus.data[i]<-sum(locs.in.gset)
      gset.anns$genes.with.locus.data[i]<-paste(gene.locs$gene.label[locs.in.gset],collapse=" // ")
      gset.anns$all.genes.matched[i]<-all(is.element(gset.data$gene.id[this.gset],
                                                     gene.locs$gene.id[locs.in.gset]))
      if (!gset.anns$all.genes.matched[i])
      {
         this.gset.data<-gset.data[this.gset,]
         unmatched<-!is.element(this.gset.data$gene.id,gene.locs$gene.id)
         gset.anns$unmatched.genes[i]<-paste(this.gset.data$gene.id[unmatched],collapse=" // ")

      }
   }

   return(gset.anns)
}

###############################################
# Function: compute.hit.pvals.one.gset
# Purpose: compute the p-values for one gene set
# Arguments: lesion.locs - data.frame with one row per lesion, includes columns for subj.ID, chrom, loc.start, and loc.end
#            gene.locs - data.frame with one row per gene, includes columns for chrom, loc.start, and loc.end
#            chrom.sizes - data.frame with one row per chromosome, includes columns for chrom and size
# Returns: a list with two components
#          cohort.res - data.frame with cohort-level statistical results
#          subj.res - data.frame with subject-specific statistical results
#############################################################################  

compute.hit.pvals.one.gset<-function(lesion.locs,gene.locs,chrom.sizes,
                                     edge.adjustment=T,quiet=T)

{
   if(is.null(lesion.locs$subj.ID)) stop("lesion.locs does not have a subj.ID column.")

   ##print(head(gene.locs))
   
   hit.stats<-compute.hit.stats.one.gset(lesion.locs,gene.locs,quiet)
   hit.probs<-pr.stats.cohort.one.gset(lesion.locs,gene.locs,chrom.sizes,
                                       quiet=quiet,edge.adjustment=edge.adjustment)

   # Initialize subject-specific p-values
   nsubj<-dim(hit.stats)[1]
   pval.cc.nlesions.that.hit.gset<-rep(NA,nsubj)  # Full convolution model, no of lesions that overlap gene-set
   pval.cc.tot.overlaps<-rep(NA,nsubj)            # Full convolution model, total number of overlaps 

   
   # initialize convolutions for number of subjects with a hit
   pmf.nsubj.with.hit<-list(n=0,p=1)

   
   ##print(dim(hit.stats))
   ##print(head(hit.stats))


   for (i in 1:nsubj)  # Compute each p-value for each subject
   {
      #message(paste("  Computing p-values for subject",i,"of",nsubj,":",date()))
      pval.cc.nlesions.that.hit.gset[i]<-discrete.dist.pvalue(hit.stats$n.lesions[i],
                                                              hit.probs$subj.conv.models[[i]]$pr.cc.nlesions.that.hit.gset)

      pval.cc.tot.overlaps[i]<-discrete.dist.pvalue(hit.stats$n.overlaps[i],
                                                    hit.probs$subj.conv.models[[i]]$pr.cc.total.overlaps)


      pr.this.subject.has.hit<-sum(hit.probs$subj.conv.models[[i]]$pr.cc.nlesions.that.hit.gset$p[(hit.probs$subj.conv.models[[i]]$pr.cc.nlesions.that.hit.gset$n>0)])

      pr.this.subject.has.hit<-max(0,min(1,pr.this.subject.has.hit))  # prevent numerical problems for probs within machine value of 0 or 1
      

      pmf.nsubj.with.hit<-conv.disc(c(0,1),c(1-pr.this.subject.has.hit,pr.this.subject.has.hit),
                                    pmf.nsubj.with.hit$n,pmf.nsubj.with.hit$p)

   }


   subj.res<-cbind.data.frame(hit.stats,
                              p.overlaps=pval.cc.tot.overlaps,
                              p.lesions=pval.cc.nlesions.that.hit.gset)


   cohort.stats<-unlist(colSums(hit.stats[,c("n.lesions","n.overlaps")]))
   

   nsubj.with.hit<-sum(hit.stats$n.overlaps>0)

   cohort.cc.pval.nlesions.that.hit.gset<-discrete.dist.pvalue(cohort.stats["n.lesions"],hit.probs$cohort.pr.cc.nlesions.that.hit.gset)

   cohort.cc.pval.tot.overlaps<-discrete.dist.pvalue(cohort.stats["n.overlaps"],hit.probs$cohort.pr.cc.total.overlaps)



   pval.con.nsubj.with.hit<-discrete.dist.pvalue(nsubj.with.hit,pmf.nsubj.with.hit)



   cohort.res0<-c(n.overlaps=cohort.stats["n.overlaps"],
                  p.overlaps=cohort.cc.pval.tot.overlaps,

                  n.lesions=cohort.stats["n.lesions"],
                  p.lesions=cohort.cc.pval.nlesions.that.hit.gset,


                  n.subjects=nsubj.with.hit,
                  p.subjects=pval.con.nsubj.with.hit
                  )
   
   names(cohort.res0)=c("n.overlaps","p.overlaps",
                        "n.lesions","p.lesions",
                        "n.subjects","p.subjects")



   return(list(subj.res=subj.res,
               cohort.res=cohort.res0))


}

compute.hit.stats.one.gset<-function(lesion.locs,gene.locs,quiet=T)

{
   if(is.null(lesion.locs$subj.ID)) stop("lesion.locs does not have a subj.ID column.")
   if(!quiet) message(paste(" Computing gene-hit statistics:",date()))

   # Initialize result vectors
   uniq.subjID<-sort(unique(lesion.locs$subj.ID))
   n.subj<-length(uniq.subjID)
   nlesions.that.hit.gset<-rep(0,n.subj)
   tot.overlaps<-rep(0,n.subj)
   names(tot.overlaps)<-names(nlesions.that.hit.gset)<-uniq.subjID

   # Subset on chromosomes that occur in both the lesion.locs and gene.locs
   relevant.lesions<-is.element(lesion.locs$chrom,gene.locs$chrom)
   lesion.locs<-lesion.locs[relevant.lesions,]
   relevant.genes<-is.element(gene.locs$chrom,lesion.locs$chrom)
   gene.locs<-gene.locs[relevant.genes,]
   n.genes<-dim(gene.locs)[1]


   # if any lesions occur on gene-set chromosomes, then compute
   ghit.mtx<-matrix(0,n.subj,n.genes)
   rownames(ghit.mtx)<-uniq.subjID
   colnames(ghit.mtx)<-gene.locs$gene.label
   if(any(relevant.lesions))
   {
     n.lesions<-sum(relevant.lesions)
     n.genes<-sum(relevant.genes)
     for (i in 1:n.lesions)
     {
        gene.overlap<-(gene.locs$chrom==lesion.locs$chrom[i])&
                      (gene.locs$loc.start<=lesion.locs$loc.end[i])&
                      (gene.locs$loc.end>=lesion.locs$loc.start[i])
        ghit.mtx[lesion.locs$subj.ID[i],]<-ghit.mtx[lesion.locs$subj.ID[i],]+gene.overlap
        n.overlap<-sum(gene.overlap)
        tot.overlaps[lesion.locs$subj.ID[i]]<-tot.overlaps[lesion.locs$subj.ID[i]]+n.overlap
        nlesions.that.hit.gset[lesion.locs$subj.ID[i]]<-nlesions.that.hit.gset[lesion.locs$subj.ID[i]]+(n.overlap>0)

     }

   }

   ghit.res<-cbind.data.frame(subj.ID=uniq.subjID,
                              n.lesions=nlesions.that.hit.gset,
                              n.overlaps=tot.overlaps,
                              ghit.mtx)

   rownames(ghit.res)<-uniq.subjID


   return(ghit.res)

}



pr.stats.cohort.one.gset<-function(lesion.locs,             # data.frame with columns subj.ID, chrom, loc.start, loc.end
                                   gene.locs,               # data.frame with columns chrom, loc.start, loc.end
                                   chrom.sizes,             # data.frame with columns chrom and size in basepairs
                                   edge.adjustment=T,       # logical that indicates whether to circularize chromosome to eliminate edge effect 
                                   quiet=F)                 # display progress messages?  TRUE = No, FALSE = YES

{
   # Some basic checking for valid input
   ##print(chrom.sizes)
   if (!all(is.element(lesion.locs$chrom,chrom.sizes$chrom))) stop("All chromosomes in lesion.locs must appear in chrom.sizes.")
   if (!all(is.element(gene.locs$chrom,chrom.sizes$chrom))) stop("All chromosomes in gene.locs must appear in chrom.sizes.")

   # Identify and count unique subjects
   uniq.subj<-sort(unique(lesion.locs$subj.ID))
   n.subj<-length(uniq.subj)

   ##print(head(gene.locs))
   # Limit gene.locs to those chroms that appear in lesion.locs
   gene.locs<-gene.locs[is.element(gene.locs$chrom,lesion.locs$chrom),]
   ##print(head(gene.locs))
   
   # Compute convolution models for each subject
   if (!quiet) message(paste(" Computing Convolution Models for Each Subject:",date()))


   subj.conv.models<-vector("list",n.subj)
   cohort.pr.cc.nlesions.that.hit.gset<-list(n=0,p=1)
   cohort.pr.cc.total.overlaps<-list(n=0,p=1)



   for (i in 1:n.subj)  # Loop over subjects
   {
      if (!quiet) message(paste(" Working on subject",uniq.subj[i],"(",i,"of",n.subj,"):",date()))
      subject.lesions<-lesion.locs[is.element(lesion.locs$subj.ID,uniq.subj[i]),]
      ##print(subject.lesions)
      subj.conv.models[[i]]<-pr.stats.one.subject(subject.lesions,gene.locs,chrom.sizes,quiet=quiet,
                                                  edge.adjustment=edge.adjustment)
      


      cohort.pr.cc.nlesions.that.hit.gset<-conv.disc(cohort.pr.cc.nlesions.that.hit.gset$n,
                                                      cohort.pr.cc.nlesions.that.hit.gset$p,
                                                      subj.conv.models[[i]]$pr.cc.nlesions.that.hit.gset$n,
                                                      subj.conv.models[[i]]$pr.cc.nlesions.that.hit.gset$p)

      cohort.pr.cc.total.overlaps<-conv.disc(cohort.pr.cc.total.overlaps$n,
                                             cohort.pr.cc.total.overlaps$p,
                                             subj.conv.models[[i]]$pr.cc.total.overlaps$n,
                                             subj.conv.models[[i]]$pr.cc.total.overlaps$p)

   }



   names(subj.conv.models)<-uniq.subj
   return(list(cohort.pr.cc.nlesions.that.hit.gset=cohort.pr.cc.nlesions.that.hit.gset,
               cohort.pr.cc.total.overlaps=cohort.pr.cc.total.overlaps,
               subj.conv.models=subj.conv.models))

}

pr.stats.one.subject<-function(lesion.locs,                # data frame with columns chrom, loc.start, loc.end
                               gene.locs,                  # data frame with columns chrom, loc.start, loc.end
                               chrom.sizes,                # data frame with columns chrom and size in basepairs
                               edge.adjustment=T,          # indicates whether to circularize chromosome to eliminate edge effects
                               quiet=T)                    # display progress messages? TRUE = No, FALSE = YES
                               

{
   # Some basic checking for valid input
   ##print(head(lesion.locs))
   ##print(head(chrom.sizes))
   if (any(!is.element(lesion.locs$chrom,chrom.sizes$chrom))) stop("All chromosomes in lesion.locs must appear in chrom.sizes.")
   if (any(!is.element(gene.locs$chrom,chrom.sizes$chrom))) stop("All chromosomes in gene.locs must appear in chrom.sizes.")

   ngenes<-dim(gene.locs)[1]  # number of genes

   # Limit analysis to chromosomes with some lesions and some gene-set genes
   lesion.chrom<-is.element(chrom.sizes$chrom,lesion.locs$chrom)
   gene.chrom<-is.element(chrom.sizes$chrom,gene.locs$chrom)
   keep.chroms<-lesion.chrom&gene.chrom

   # Return result for trivial case of no gene-set chromosomes have a lesion
   if (!any(keep.chroms)) return(list(pr.cc.nlesions.that.hit.gset=list(n=0,p=1),
                                      pr.cc.total.overlaps=list(n=0,p=1)))

   chrom.sizes<-chrom.sizes[keep.chroms,]
   nchroms<-dim(chrom.sizes)[1]

   # Initialize results
   pr.cc.nlesions.that.hit.gset<-list(n=0,p=1)     # Initialize the convolution- of convolutions distribution for the number of lesions that hit a gene-set gene
   pr.cc.total.overlaps<-list(n=0,p=1)             # Initialize the convolution-convolution distribution for the total number of overlaps
   # Loop over chromosomes
   for (i in 1:nchroms)  # loop over chromosomes
   {
      if (!quiet) message(paste("  Working on chromosome",chrom.sizes$chrom[i],":",date()))

      # Extract data for chromosome i
      chrom.lesions<-is.element(lesion.locs$chrom,chrom.sizes$chrom[i])
      chrom.genes<-is.element(gene.locs$chrom,chrom.sizes$chrom[i])

      gene.start.end<-gene.locs[chrom.genes,c("loc.start","loc.end")]
      lesion.sizes<-lesion.locs$loc.end[chrom.lesions]-lesion.locs$loc.start[chrom.lesions]+1

      # Compute probability distributions for chromosome i
      phit.chrom<-pr.hits.one.chromosome(lesion.sizes,gene.start.end,chrom.sizes$size[i],
                                         edge.adjustment=edge.adjustment)
      pstat.chrom<-pr.stats.one.chromosome(phit.chrom)

      # Update convolutions
      pr.cc.nlesions.that.hit.gset<-conv.disc(pr.cc.nlesions.that.hit.gset$n,
                                              pr.cc.nlesions.that.hit.gset$p,
                                              pstat.chrom$pr.nlesions.that.hit.gset$n,
                                              pstat.chrom$pr.nlesions.that.hit.gset$p)

      pr.cc.total.overlaps<-conv.disc(pr.cc.total.overlaps$n,
                                      pr.cc.total.overlaps$p,
                                      pstat.chrom$pr.total.overlaps$n,
                                      pstat.chrom$pr.total.overlaps$p)

      nlesion<-length(phit.chrom)

   }

   return(list(pr.cc.nlesions.that.hit.gset=pr.cc.nlesions.that.hit.gset,
               pr.cc.total.overlaps=pr.cc.total.overlaps))
}


###################################
# Function: pr.stats.one.chromosome
# Purpose: Compute the probability of various hit statistics under the GRIN model for one chromosome
# Arguments: pr.hits - the result of pr.hits.one.chromosome
# Returns: a list with the probability distributions for various stats
###################################

pr.stats.one.chromosome<-function(pr.hits)

{
   pr.nlesions.that.hit.gset<-list(n=0,p=1)     # Initialize the distribution for the number of lesions that hit a gene-set gene
   mean.prob.one.lesion.hits.gset<-0            # Initialize the value of the mean.prob.
   pr.total.overlaps<-list(n=0,p=1)             # Initialize the distribution for the total number of overlaps
   sum.pr.gset.hit<-0

   nlesions<-length(pr.hits)
   for (i in 1:nlesions)  # Loop over lesions
   {
      # Convolve the distribution with that of lesion i
      p.gset.hit<-pr.hits[[i]]$p.gset.hit
      if (p.gset.hit<0) p.gset.hit<-0 # For numerical stability
      if (p.gset.hit>1) p.gset.hit<-1
      sum.pr.gset.hit<-sum.pr.gset.hit+p.gset.hit

      #message("update nlesions")
      pr.nlesions.that.hit.gset<-conv.disc(pr.nlesions.that.hit.gset$n,
                                           pr.nlesions.that.hit.gset$p,
                                           c(0,1),c(1-p.gset.hit,p.gset.hit))

      #message("update overlaps")
      pr.total.overlaps<-conv.disc(pr.total.overlaps$n,
                                   pr.total.overlaps$p,
                                   pr.hits[[i]]$p.ngene.hit$n,
                                   pr.hits[[i]]$p.ngene.hit$p)

      # Add up the probability that each lesion hits the gene-set for Cheng's binomial model
      mean.prob.one.lesion.hits.gset<-mean.prob.one.lesion.hits.gset+pr.hits[[i]]$p.gset.hit
   }

   return(list(pr.nlesions.that.hit.gset=pr.nlesions.that.hit.gset,
               pr.total.overlaps=pr.total.overlaps,
               nlesions=nlesions,sum.pr.gset.hit=sum.pr.gset.hit))
}


#####################################
# Function: pr.hits.one.chromosome
# Purpose: For each lesion size, determine the probability of each possible subset of hit genes under the GRIN model
# Arguments: lesion.sizes - vector of lesion sizes in base pairs
#            gene.start.end - 2-column matrix with start and end position for each gene (row)
#            chrom.size - scalar with chromosome size in base pairs
# Returns: a vector of lists, each entry gives the results for the lesion as a list with the following componments
#          hit.mtx - a matrix that indicates (0/1) whether each interval in the piecewise function (column) overlaps each gene (row)
#          p.mtx - a vector of associated probabilities
############################################

pr.hits.one.chromosome<-function(lesion.sizes,      # vector of lesion sizes in base pairs
                                 gene.start.end,    # 2-column matrix with start and end positions for genes
                                 chrom.size,        # scalar: chromosome size in base pairs
                                 relative.sizes=F,  # indicates whether lesion.sizes are relative to the size of the chromosome
                                 edge.adjustment=T) # indicates whether to circularize chromosome to eliminate edge effects

{
   ##print(summary(lesion.sizes))
   ##print(chrom.size)

   if(length(chrom.size)>1) stop("chrom.size must be a scalar")
   if((any(lesion.sizes<1))&!relative.sizes) stop("relative.sizes must be TRUE if some lesion.size < 1")
   if(relative.sizes) lesion.sizes<-pmax(1,round(chrom.size*lesion.sizes))   
   #if(any(lesion.sizes>chrom.size)) stop("Each entry of lesion.sizes must be <= chrom.size")
   if(any(gene.start.end>chrom.size)) stop("The loc.start and loc.end of each gene must be <= chrom.size")


   n.lesion<-length(lesion.sizes)        # Number of lesions
   n.genes<-dim(gene.start.end)[1]       # Number of genes in the gene.set
   pr.hits<-vector("list",n.lesion)      # initialize result
   names(pr.hits)<-paste("lesion",1:n.lesion,sep="_")

   # Convert to positions to (0,1) interval
   gene.start.end$loc.start<-gene.start.end$loc.start/chrom.size
   gene.start.end$loc.end<-gene.start.end$loc.end/chrom.size
   lesion.sizes<-lesion.sizes/chrom.size
   chrom.size<-1


   # Loop over lesions
   for (i in 1:n.lesion)
   {
      if (lesion.sizes[i]==chrom.size)  # Take care of this trivial case
      {
        pr.hits[[i]]$gene.hit<-matrix(1,1,n.genes)    # Only possible subset is all genes
        pr.hits[[i]]$p<-1                             # probability of hitting this subset is 1
      }
      else
      {
        # Determine jump-points in the piecewise constant function
        gene.hit.intervals<-gene.overlap.intervals(gene.start.end[,c("loc.start","loc.end")],
                                                   lesion.sizes[i],
                                                   chrom.size,
                                                   edge.adjustment)
        if (edge.adjustment) segment.endpts=sort(unique(c(0,unlist(gene.hit.intervals),1)))
        else  segment.endpts<-sort(unique(c(0,unlist(gene.hit.intervals),1-lesion.sizes[i])))                         # Now use gene.hit.intervals to define segment endpoints
        pr.hits[[i]]$brks.pwc<-segment.endpts                                                                   # Keep the breakpoints of the piecewise constant function for lesion i
        npts<-length(segment.endpts)                                                                            # number of endpoints

        # Generate matrix indicating whether each piecewise constant function interval overlaps each gene
        pr.hits[[i]]$ghit.mtx<-matrix(NA,npts-1,n.genes)
        for (j in 1:(npts-1))
           pr.hits[[i]]$ghit.mtx[j,]<-(segment.endpts[j]<gene.hit.intervals[,2])*
                                      (segment.endpts[j+1]>gene.hit.intervals[,1])                              # indicates whether each pwc interval (row) overlaps a gene (column)
        rownames(pr.hits[[i]]$ghit.mtx)<-paste("pwc_interval_",1:(npts-1),sep="")
        colnames(pr.hits[[i]]$ghit.mtx)<-paste("gene_",1:n.genes,sep="")


        segment.probs<-diff(segment.endpts)/(1-lesion.sizes[i]) # probabilities are lengths of the intervals definining the piece-wise constant function
        pr.hits[[i]]$p.row.ghit.mtx<-segment.probs              # save in the object

        ngene.hit<-rowSums(pr.hits[[i]]$ghit.mtx)               # Compute the number of genes that overlap each interval of the piecewise constant function


        pr.hits[[i]]$p.gset.hit<-sum(segment.probs*(ngene.hit>0))  # probability that lesion i hits at least one gene in the gene-set

        uniq.ngene.hit<-sort(unique(ngene.hit))
        p.ngene.hit<-rep(NA,length(uniq.ngene.hit))
        for (j in 1:length(uniq.ngene.hit)) p.ngene.hit[j]<-sum(segment.probs*(ngene.hit==uniq.ngene.hit[j]))

        pr.hits[[i]]$p.ngene.hit<-list(n=uniq.ngene.hit,p=p.ngene.hit)
      } # end else
   } # end loop over lesions
   return(pr.hits)  # list-vector, one entry per lesion, each component includes the gene.hit matrix that indicates whether each piecewise-constant interval (row) overlaps each gene (column) and a vector of associated probabilities
}

##########################################
# Function: gene.overlap.intervals
# Purpose: Determine the intervals for the start position of a lesion to overlap a set of genes
#########################################

gene.overlap.intervals=function(gene.start.end,      # two column matrix with start and end of genes
                                lesion.size,         # scalar
                                chrom.size,          # scalar
                                edge.adjustment=T)   # logical indicating whether to circularize chromosome to eliminate edge effects
  
{
  
  gene.ints=gene.start.end
  gene.ints[,1]=gene.ints[,1]-lesion.size
  neg.start=which(gene.ints[,1]<0)
  
  
  if (edge.adjustment)
  {
    
    for (i in neg.start)
    {
      extend=gene.ints[i,1]
      gene.ints[i,1]=0
      rbind(gene.ints,
            c(chrom.size+extend,chrom.size))
    }
  }
  else
  {
    gene.ints[neg.start,1]=0
    too.far=gene.ints[,2]>(chrom.size-lesion.size)
    gene.ints[too.far,2]=chrom.size-lesion.size
  }
  return(gene.ints)
}


###################################
# Function: discrete.dist.pvalue
# Purpose: Compute the p-value for a discrete distribution
# Arguments: n - observed number of events
# discrete.dist: list with components n (number of events) and p (null probabilities)
#####################################

discrete.dist.pvalue<-function(n,discrete.dist)

{
   n.tail<-(discrete.dist$n>=n)
   pval<-sum(discrete.dist$p[n.tail])
   return(pval)
}

#############################
# Function: conv.disc
# Purpose: Convolve two discrete distributions
# Arguments: n1,p1: number and probabilities for distribution 1
#            n2,p2: number and probabilities for distribution 2
# Returns: list with n = number and p = probability for the convolution
###############################

conv.disc<-function(n1,p1,n2,p2)

{
  # Take logs for numerical stability
  log.p1<-log(p1)
  log.p2<-log(p2)
  
  ##print(cbind(n1,p1,log.p1))
  ##print(cbind(n2,p2,log.p2))
  
  n.new<-(min(n1)+min(n2)):(max(n1)+max(n2))
  min.new<-min(n.new)
  length.new<-length(n.new)
  p.new<-rep(0,length.new)

  short<-which.min(c(length(p1),length(p2)))
                                                                              
                                                                                
  if (short==1)
  {
     short.ind<-(1:length(n1))
     for (i in short.ind)
     {
        this.ind<-n1[i]+n2-min.new+1
        p.new[this.ind]<-p.new[this.ind]+exp(log.p2+log.p1[i])
     }
  }

  if (short==2)
  {
     short.ind<-(1:length(n2))
     for (i in short.ind)
     {
        this.ind<-n2[i]+n1-min.new+1
        p.new[this.ind]<-p.new[this.ind]+exp(log.p1+log.p2[i])
     }
  }

  if (any(is.na(c(p.new,n.new))))
  {
     ##print(cbind(n1,p1,log.p1))
     ##print(cbind(n2,p2,log.p2))
     ##print(cbind(n.new,p.new))
     stop("NAs in conv.disc")  
  }


  # Eliminate unnecessary items of result
  zero.prob<-(p.new==0)
  p.new<-p.new[!zero.prob]
  n.new<-n.new[!zero.prob]
  
  # return result
  return(list(n=n.new,p=p.new))
}

#############################################################
#> BEGIN add.gene.anns
#> \title{add gene annotation}
#> \description{annotate a set of interval (genomic lesions) or point loci (microarray markers) with nearby or overlapping genes}
                                          #> \arguments{
add.gene.anns<-function(res.data,         #> \item{res.data}{a data.frame or name of a tab-delimited text file with locations of the loci to be annotated} 
                        res.chrom,        #> \item{res.chrom}{name or numeric index of chromosome column in res.data}
                        res.start,        #> \item{res.start}{name or numeric index of start position column in res.data}
                        res.end,          #> \item{res.end}{name or numeric index of end position column in res.data}
                        gene.data,        #> \item{gene.data}{a data.frame or name of a tab-delimited text file with one row per gene, columns for gene.id, gene.label, chrom, loc.start, loc.end}
                        gene.id,          #> \item{gene.id}{name or numeric index of gene identifier column in gene.data}
                        gene.chrom,       #> \item{gene.chrom}{name or numeric index of chromosome column in gene.data}
                        gene.start,       #> \item{gene.start}{name or numeric index of start position in gene.data}
                        gene.end,         #> \item{gene.end}{name or numeric index of end position in gene.data}
                        annres.file=NULL, #> \item{annres.file}{name of output annotated result file}
                        gene.type="gene", #> \item{gene.type}{name of the "type of gene", e.g. "gene" or "miRNA"}
                        bp.win=5000,      #> \item{bp.win}{size of a matching window in base pairs}
                        max.genes=15,     #> \item{max.genes}{maximum number of genes to print the gene symbol}
                        return.result=T)  #> \item{return.result}{indicates whether to return the results to R or not}
                                          #> }
{
   if (is.character(res.data))  res.data<-read.table(res.data,sep="\t",as.is=T,header=T)
   if (is.character(gene.data)) gene.data<-read.table(gene.data,sep="\t",as.is=T,header=T,quote="")
   
   dup.genes<-duplicated(gene.data[,gene.id])
   gene.data<-gene.data[!dup.genes,]
   print(gene.data[1:5,])
   
   res.chr<-chrom.as.number(res.data[,res.chrom])
   print(table(res.chr))
   gene.chr<-chrom.as.number(gene.data[,gene.chrom])
   print(table(gene.chr))
  

   on.res.chr<-is.element(gene.chr,res.chr)
   print(table(on.res.chr))
   gene.data<-gene.data[on.res.chr,]
   gene.chr<-gene.chr[on.res.chr]

   ngenes<-dim(gene.data)[1]
   nres<-dim(res.data)[1]
   print(nres)
   gene.anns<-rep("",nres)
   ngenes.mtch<-rep(0,nres)  # No. of genes that overlap each "result" (segment or SNP)
   for (i in 1:ngenes)       # Loop over genes
   {
      chr.mtch<-is.element(res.chr,gene.chr[i])   # Check for matching chromosome
      overlap1<-rep(F,nres)
      overlap1[chr.mtch]<-res.data[chr.mtch,res.start]<=(gene.data[i,gene.end]+bp.win) # Check that the start of the result (segment or SNP) is before the end of the gene
      overlap2<-rep(F,nres)
      overlap2[chr.mtch]<-res.data[chr.mtch,res.end]>=(gene.data[i,gene.start]-bp.win) # checks that the start of the gene is before the end of the result (segment or SNP)
      pos.mtch<-chr.mtch&overlap1&overlap2                                             # Indicates for each result, whether or not the gene overlaps
      ngenes.mtch[pos.mtch]<-ngenes.mtch[pos.mtch]+1                                   # Increments
      if (any(pos.mtch)) gene.anns[pos.mtch]<-paste(gene.anns[pos.mtch],gene.data[i,gene.id])  # Keeps the gene names
   }
   gene.anns[ngenes.mtch>max.genes]<-paste(">",max.genes)  # Reduces the lengthy gene lists to "> max.genes"
   gene.res<-cbind.data.frame(ngenes.mtch,gene.anns)
   names(gene.res)<-c(paste("n.",gene.type,sep=""),
                      paste(gene.type,".list",sep=""))

   fin.res<-cbind.data.frame(res.data,gene.res)
   if (!is.null(annres.file))   write.table(fin.res,annres.file,sep="\t",col.names=T,row.names=F,quote=T)
   if (return.result) return(fin.res)

}
#> \examples{
#> data(ETP.lesion.data)
#> data(hg18.gene.annotation.data)
#> data(hg18.chrom.sizes.data)
#> #ann.res<- add.gene.anns(ETP.lesion.data,2,3,4,hg18.gene.annotation.data,"gene.label","chrom",
#>                      # "loc.start","loc.end",max.genes=20)  #commented out example
#> }

#> \value{data.frame with res.data and the gene annotations for the leision data.}
#> END add.gene.anns




#############################################################
#> BEGIN robust.fdr
#> \title{Robust False Discovery Rate}
#> \description{Use p-values to estimate or control the false discovery rate(FDR) and related multiple testing error measure.}
                                        #> \arguments{
robust.fdr<-function(p,                 #> \item{p}{vector of p-values from the analysis}
                     sides=1,           #> \item{sides}{indicate whether p-values are 1-sided (set sides=1) or 2-sided (set sides=2), default=1}
                     p2=1-p,            #> \item{p2}{for one-sided testing, p-values from testing the "other alternative", default=1-p}
                     discrete=F,        #> \item{discrete}{indicates (T/F) whether p-values are discrete}
                     use8=T)            #> \item{use8}{indicates whether the constant 8 should be used if p-values are discrete, see Pounds and Cheng (2006) for more details} }
{
	if (all(p==1)) return(list(p=p,
                             fdr=rep(1,length(p)),
                             q=rep(1,length(p)),
                             cdf=rep(1,length(p)),
                             loc.fdr=rep(1,length(p)),
                             fp=rep(1,length(p)),
                             fn=rep(1,length(p)),
                             te=rep(1,length(p)),
                             pi=1,ord=1:length(p)))
  m<-length(p)
	ord<-order(p)
	pcumtab<-cumsum(table(p))/m
	F05<-mean(p<=0.5)	
	edf<-approx(as.numeric(names(pcumtab)),pcumtab,xout=p,rule=2)$y
	if (sides==2)
	{
		pi<-min(1,2*mean(p))
		loc.fdr<-pi*p/edf
	}
	else
	{
		p.new<-2*(p*(p<=p2)+p2*(p>p2))
		pi<-min(1,2*mean(p.new))
		if (discrete) 
		{
			if (use8) pi<-min(1,8*mean(p.new))
			else
			{
				lam<-max(p[p<=0.5])
				k<-1/(lam^2+(0.5-lam)^2)
				pi<-min(k*mean(p.new),1)
			
			}
		}
		loc.fdr<-pi*p/edf
		loc.fdr[p>0.5]<-(0.5*pi+edf[p>0.5]-F05)/edf[p>0.5]
	}
  loc.fdr[loc.fdr>1]=1
	up<-unique(p[ord])
  up[up>1]=1
	ufdr<-approx(p[ord],loc.fdr[ord],xout=up)$y
  ufdr[ufdr>1]=1
  last=min(which(ufdr==1),length(ufdr))
  if (last>=2)
  {
	  res<-lts.rank.regr(up[1:last],ufdr[1:last])
	  fdr<-approx(up[1:last],res$yhat,xout=p,rule=2)$y
	  fdr[fdr>1]<-1
  }
  else fdr=rep(1,length(p))
	fp<-m*edf*fdr
	fn<-m*(F05-edf-pi*(0.5-p))*(p<0.5)*(sides==1)+(sides==2)*m*(1-edf-pi*(1-p))
	fn[fn<0]<-0
	te<-fp+fn
	q<-fdr
	q[rev(ord)]<-cummin(fdr[rev(ord)])
	
	return(list(p=p,fdr=fdr,q=q,cdf=edf,loc.fdr=loc.fdr,fp=fp,fn=fn,te=te,pi=pi,ord=ord))
}
#> \references{Pounds S, Cheng C, Li, S, Liu Z, Zhang J, Mullighan C(2013) Genomic Random Interval Models for Statistical Analysis of Genomic Lesion Data. Bioinformatics.
#>
#>
#>             Pounds and Cheng (2006) Robust estimation of the false discovery rate. Bioinformatics 22: 1979-87.}

#> \value{a list with the following components:
#>            \item{p}{the vector of p-values provided by the user}
#>            \item{fdr}{the vector of smoothed FDR estimates}
#>            \item{q}{the vector of q-values based on the smoothed FDR estimates}
#>            \item{cdf}{the vector with p-value empirical distribution function at corresponding entry of p}
#>            \item{loc.fdr}{the local (unsmoothed) FDR estimates}
#>            \item{fp}{the estimated number of false positives at p-value cutoff in p}
#>            \item{fn}{the estimated number of false negatives at p-value cutoff in p}
#>            \item{te}{the total of fp and fn}
#>            \item{pi}{the null proportion estimate}
#>            \item{ord}{a vector of indices to order the vectors above by ascending p-value} }
#> END robust.fdr


lts.rank.regr<-function(x,y)

{
	if(length(x)==2) return(list(x=x,y=y,yhat=y))
  rx<-rank(x)
	ry<-rank(y)
  #print(cbind(rx,ry))
	rfit<-ltsreg(rx,ry)
	ryhat<-rfit$fitted.values
	yhat<-approx(ry,y,xout=ryhat,rule=2)$y
	return(list(x=x,y=y,yhat=yhat))
}


#############################################################
#> BEGIN chrom.as.number
#> \title{chrom.as.number}
#> \description{Convert chrom X, Y, and MT into 23, 24, and 25, respectively to comply with that convention.}
                                   #> \arguments{
chrom.as.number<-function(chr)     #> \item{chr}{a vector of chrom } }

{
   chr<-as.character(chr)
   Xchr<-is.element(chr,"X")
   Ychr<-is.element(chr,"Y")
   MTchr<-is.element(chr,"MT")
   
   if (any(Xchr)) chr[Xchr]<-23
   if (any(Ychr)) chr[Ychr]<-24
   if (any(MTchr)) chr[MTchr]<-25
   chr<-as.numeric(as.character(chr))
   return(chr)
}

#> \value{a vector}
#> END chrom.as.number




#############################################################
#> BEGIN seg.man.plot
#> \title{generate a Manhattan plot for segment data}
#> \description{Genrate the heat map for leision data}
                                        #> \arguments{
seg.man.plot=function(seg.res,          #> \item{seg.res}{data.frame with one row per lesion, required columns are subj.ID, chrom, loc.start, loc.end and pvalue(column's name which will be plotted)}
                      chr.sizes,        #> \item{chr.sizes}{data.frame with one row per chromosome, required columns are chrom & size}
                      plot.column,      #> \item{plot.column}{the column's name which will be plotted} 
                      ...)              #> \item{...}{additional arguments to for the plot function}}
{
  #print(summary(seg.res))
  ord=order(seg.res$chrom,seg.res$loc.start)
  seg.res=seg.res[ord,]
  chr.ord=order(chr.sizes$chrom)
  chr.sizes=chr.sizes[chr.ord,]
  cum.chr.size=cumsum(as.numeric(chr.sizes[,2]))
  cum.chr.size=c(0,cum.chr.size)
  #print(cum.chr.size)
  seg.res$g.start=as.numeric(seg.res$loc.start)+as.numeric(cum.chr.size[seg.res$chrom])
  seg.res$g.end=as.numeric(seg.res$loc.end)+as.numeric(cum.chr.size[seg.res$chrom])
  seg.res$g.start=seg.res$g.start/cum.chr.size[length(cum.chr.size)]
  seg.res$g.end=seg.res$g.end/cum.chr.size[length(cum.chr.size)]
  plot(c(-0.5,10),c(0,1),xlab="",ylab="",type="n",axes=F,xlim= c(-1,10))
  rect(-0.1,0,-log10(0.05),1,col="lightgray",border="lightgray")
  nsegs=nrow(seg.res)
  for (i in 1:nsegs)
  {
   rect(-log10(seg.res[i,plot.column]),1-seg.res$g.start[i],0,1-seg.res$g.end[i],
   border = c(1:24)[seg.res$chrom[i]],col= c(1:24)[seg.res$chrom[i]] )
  }
  rect(0,0,10,1,col=NA,border="black") # box for the figure
  g.start =g.end = NA;
  for (i in unique(seg.res$chrom))
{
  this.subj=is.element(seg.res$chrom,i)
  g.start[i]= max (c(seg.res$g.start[this.subj],seg.res$g.end[this.subj]))
  g.end[i]= min (c(seg.res$g.start[this.subj],seg.res$g.end[this.subj]))
  }
  rect(-0.5,1-g.start,-0.2,1-g.end,border=c("black","white"),col= c("black","white"))
  rect(-0.5,0,-0.2,1, border="black") 
  mtext(c(0,3,6,9),side=1,at=c(0,3,6,9),cex=1,padj=-0.5)
  #mtext(c(1:22,"X","Y"),side=2,at= 1-(cum.chr.size[2:25]+cum.chr.size[1:24])/(2*cum.chr.size[25]),cex=0.5,padj=0)
  text(-0.8,1-(cum.chr.size[2:25]+cum.chr.size[1:24])/(2*cum.chr.size[25]),c(1:22,"X","Y"),cex=1)
}
#> \examples{
#> data(ETP.lesion.data)
#> data(hg18.gene.annotation.data)
#> data(hg18.chrom.sizes.data)
#> genome.res<-grin.genome(ETP.lesion.data, hg18.gene.annotation.data, hg18.chrom.sizes.data)
#> seg.man.plot(genome.res,hg18.chrom.sizes.data,"p.overlaps")
#> }
#> \references{Pounds S, Cheng C, Li, S, Liu Z, Zhang J, Mullighan C(2013) Genomic Random Interval Models for Statistical Analysis of Genomic Lesion Data. Bioinformatics.}

#> \value{NULL}
#> END seg.man.plot


#############################################################
#> BEGIN seg.heatmap
#> \title{generate a heat map for segment data}
#> \description{Genrate the heat map for leision data}
                                                       #> \arguments{
seg.heatmap=function(segs.data,                        #> \item{segs.data}{data.frame with one row per lesion, required columns are subj.ID, chrom, loc.start, loc.end and type}
                     chr.sizes=hg19.chrom.sizes.data,  #> \item{chr.sizes}{data.frame with one row per chromosome, required columns are chrom & size}
                     value.column="type",              #> \item{value.column}{the column name for the leision type}
                     color.scheme=NULL,                 #> \item{color.scheme}{a data.frame with column value (lesion type) and color}
                     ...)                              #> \item{...}{additional arguments to for the plot function}}
{
  ord=order(segs.data$chrom,segs.data$loc.start)
  segs.data=segs.data[ord,]
  chr.ord=order(chr.sizes$chrom)
  chr.sizes=chr.sizes[chr.ord,]
  cum.chr.size=cumsum(as.numeric(chr.sizes[,2]))
  cum.chr.size=c(0,cum.chr.size)
  if (is.null(color.scheme))
  {
    value = as.character (sort(unique(segs.data[,value.column])))
    color.list = c("red","blue","purple","cyan","gold","black","brown","chocolate1","magenta","yellow","green3")
    if(length(value)>length(color.list)) # recycle the color
    {color = color.list[1:length(value),1:(length(value)-length(color.list))]}
    else 
   {color = color.list[1:length(value)]}
   color.scheme =cbind(value,color)
  }
  #print(cum.chr.size)
  segs.data$g.start=as.numeric(segs.data$loc.start)+as.numeric(cum.chr.size[segs.data$chrom])
  segs.data$g.end=as.numeric(segs.data$loc.end)+as.numeric(cum.chr.size[segs.data$chrom])
  segs.data$g.start=segs.data$g.start/cum.chr.size[length(cum.chr.size)]
  segs.data$g.end=segs.data$g.end/cum.chr.size[length(cum.chr.size)]
  segs.data$color="black"
  for (i in 1:nrow(color.scheme)) 
    segs.data$color[(segs.data[,value.column]==color.scheme[i,"value"])]=color.scheme[i,"color"]
  uniq.id=sort(unique(segs.data$subj.ID))
  segs.data$size=segs.data$g.end-segs.data$g.start
  size.ord=rev(order(segs.data$size))
  segs.data=segs.data[size.ord,]
  
  plot(c(-1.1*length(uniq.id)/10,length(uniq.id)),c(0,1),axes=F,type="n",xlab="",ylab="")
  
  for (i in 1:length(uniq.id))
  {
    this.subj=is.element(segs.data$subj.ID,uniq.id[i])
    rect(i-1,1-segs.data$g.start[this.subj],i,1-segs.data$g.end[this.subj],
         col=segs.data$color[this.subj],border=segs.data$color[this.subj])
  }
  cum.chr.size=cum.chr.size/cum.chr.size[length(cum.chr.size)]
  rect(-0.75*(length(uniq.id)/10),1-cum.chr.size[1:(length(cum.chr.size)-1)],-0.25*(length(uniq.id)/10),1-cum.chr.size[2:length(cum.chr.size)],col=c("black","white")[(2:length(cum.chr.size))%%2+1],
       border="black")
  rect(-0.75*(length(uniq.id)/10),0,-0.25*(length(uniq.id)/10),1, border="black") 
  
  
  rect(0,0,length(uniq.id),1,border="black")
  sub.value = substr(value,1,5)
  #text.width= 1,
  legend(0,-0.05,fill=color,legend= sub.value ,ncol=length(value),xpd=T,xjust=0,cex=1)
  text(-1*(length(uniq.id)/10),1-(cum.chr.size[2:25]+cum.chr.size[1:24])/(2*cum.chr.size[25]),c(1:22,"X","Y"),cex=1)
}
#> \examples{
#> data(ETP.lesion.data)
#> data(hg18.gene.annotation.data)
#> data(hg18.chrom.sizes.data)
#> seg.heatmap(ETP.lesion.data,hg18.chrom.sizes.data )
#> }
#> \references{Pounds S, Cheng C, Li, S, Liu Z, Zhang J, Mullighan C(2013) Genomic Random Interval Models for Statistical Analysis of Genomic Lesion Data. Bioinformatics.}
#> \value{NULL}
#> END seg.heatmap



#########################################
#> BEGIN grin.analysis
#> \title{Generate a complete analysis:grin.genes, grin.genome, grin.gset for the lesion data}
#> \description{The function take the input lesion data and perform grin.genes, grin.genome, grin.gsets, and save those results to result directory.}
                                                              #> \arguments{
grin.analysis=function(lesion.data,                           #> \item{lesion.data}{data.frame with one row per lesion, columns for subj.ID, chrom, loc.start, loc.end and type}
                     geneloc.data=hg19.gene.annotation.data,  #> \item{geneloc.data}{data.frame with one row per gene, columns for gene.id, gene.label, chrom, loc.start, loc.end. The default is hg19.gene.annotation.data}
                     gset.data=KEGG.gset.data,                #> \item{gset.data}{data.frame with one row per assignment of gene to gene-set, columns for gset.id, gset.label, and gene.id. The default is KEGG.gset.data}
                     chrom.data=hg19.chrom.sizes.data,        #> \item{chrom.data}{data.frame with one row per chromosome, columns for chrom and size. The default is hg19.chrom.sizes.data}    
                     prefix="",                               #> \item{prefix}{prefix for the result file}  
                     result.dir=NULL)                         #> \item{result.dir}{the working directory to store the results}    
                                                              #> }
{
# data checking
  message(paste("grin.analysis started:",date()))
  if (!all(is.element(c("subj.ID","chrom","loc.start","loc.end","type"),colnames(lesion.data)))) stop("lesion.locs must have the subj.ID, chrom, loc.start, and loc.end and type columns.") 
  if (!all(is.element(c("gene.id","gene.label","chrom","loc.start","loc.end"),colnames(geneloc.data)))) stop("gene.locs must have the gene.id, chrom, loc.start, and loc.end columns.")
  if (!all(is.element(c("chrom","size"),colnames(chrom.data)))) stop("chrom size data must have the chrom and size columns.")
  if (!all(is.element(c("gset.id","gene.id"),colnames(gset.data)))) stop("gset.data must have the gset.id and gene.id columns.")

if (!is.null(result.dir))
{  subres.info=file.info(result.dir)
  if (is.na(subres.info$isdir)) {stop(paste(result.dir,"directory you entered doesn't exist"))}
  else if (!is.na(subres.info$isdir) & subres.info$isdir){ message(paste(result.dir,"is a working directory"))}
  else
  {
   result.dir <- getwd()
   message(paste(result.dir,"is not a working directory"))
   message("The result will be stored in the currrent working directory!")
  }
}
else
{
   result.dir <- getwd()
   message("The result will be stored in the currrent working directory!")
}  
  result.dir <- paste(result.dir,"/",sep="")
 
  heatmap<-paste(result.dir,prefix,"heatmap.pdf",sep="")
  pdf(heatmap,height=10,width=8)
  seg.heatmap(lesion.data,chrom.data,cex.lab=1.8,cex.axis=1.8,cex.main=2,cex.sub=1.6)
  dev.off()
  message(paste("Heatmap completed:",date()))
  sort.pvalue="p.subjects" 

#grin.gene analsis
##################################################################################
gene.res<-grin.genes(lesion.data,geneloc.data,chrom.data)
gene.res<-gene.res[order(gene.res[,"p.subjects"]),]
gene.res.file<-paste(result.dir,prefix,"gene-grin-results.csv",sep="")
write.csv(gene.res, gene.res.file,row.names=F)

manhattan.gene.nsubj<-paste(result.dir,prefix,"gene-nsubj-manhattan-plot.pdf",sep="")
pdf(manhattan.gene.nsubj,height=10,width=13)
seg.man.plot(gene.res, chrom.data,"p.subjects",cex.lab=1.8,cex.axis=1.8,cex.main=2,cex.sub=1.6)
dev.off() 

manhattan.gene.nlesion<-paste(result.dir,prefix,"gene-nlesion-manhattan-plot.pdf",sep="")
pdf(manhattan.gene.nlesion,height=10,width=13)
seg.man.plot(gene.res, chrom.data,"p.overlaps",cex.lab=1.8,cex.axis=1.8,cex.main=2,cex.sub=1.6)
dev.off()
message(paste("grin.gene completed:",date()))

#grin.gset analysis
#############################################################################
gset.res<-grin.gsets(lesion.data,geneloc.data,gset.data,chrom.data)
gset.res.file<-paste(result.dir,prefix,"gene-set-grin-results.csv",sep="")
write.csv(gset.res,gset.res.file,row.names=F,quote=T)
message(paste("grin.gset completed:",date()))

#grin.genome analysis
##############################################################################
genome.res<-grin.genome(lesion.data,geneloc.data,chrom.data)
ann.ord<-order(-genome.res$n.subjects,genome.res[,sort.pvalue])
genome.res<-genome.res[ann.ord,c(1:14,dim(genome.res)[2]-1,dim(genome.res)[2],15:dim(genome.res)[2]-2)]
genome.res.file<-paste(result.dir,prefix,"genome-grin-results.csv",sep="")
write.csv(genome.res,genome.res.file,row.names=F,quote=T)

manhattan.genome.nsubj<-paste(result.dir,prefix,"genome-nsubj-grin-manhattan-plot.pdf",sep="")
pdf(manhattan.genome.nsubj,height=10,width=13)
seg.man.plot(genome.res, chrom.data,"p.subjects",cex.lab=1.8,cex.axis=1.8,cex.main=2,cex.sub=1.6)
dev.off()

manhattan.genome.nlesion<-paste(result.dir,prefix,"genome-nlesion-grin-manhattan-plot.pdf",sep="")
pdf(manhattan.genome.nlesion,height=10,width=13)
seg.man.plot(genome.res, chrom.data,"p.overlaps",cex.lab=1.8,cex.axis=1.8,cex.main=2,cex.sub=1.6)
dev.off()

message(paste("grin.analysis completed:",date()))

  if (!is.null(result.dir))
  { message( paste("The result will be stored in the directory:",result.dir,sep=" "))}
  else
  { result.dir <- getwd()
   message( paste("The result will be stored in the directory:",result.dir,sep=" ")) }  
}
#> \examples{
#> data(HYPO.lesion.data)
#> #grin.analysis(HYPO.lesion.data,prefix="HYPO-")
#> }
#> \references{Pounds S, Cheng C, Li, S, Liu Z, Zhang J, Mullighan C(2013) Genomic Random Interval Models for Statistical Analysis of Genomic Lesion Data. Bioinformatics.
#>
#>
#>             Pounds and Cheng (2006) Robust estimation of the false discovery rate. Bioinformatics 22: 1979-87.}
#> \value{NULL}
#> END grin.analysis
